from time import *
from math import ceil

############################################################################# column properties
class genericColumn(object):
   """
   This class collection represents all the possible columns: including the caption of the columns and the decorators to be applied to the data
   """
   colName='colName'        # column title
   attrNames=["attrNames",] # name of the key in DAS dictionary output
   def __init__(self,path='',attrNames=[]):
     if len(path)>0: self.path=path
     if len(attrNames)>0: self.attrNames=attrNames
     
   def decorator(self,values): 
     """
     Operations to be performed on the values in order to print them
     """
     if len(values)==1: return str(values[0]) 
     return str(values)
    
   def humanFilesize(self,size):
    """
    Convert size in Bytes to more human readable format
    """
    if size < 1024:
        return ("%d B") % (size)
    elif size < (1024 * 1024):
        return ("%.2f KB") % (float(size) / (1024))
    elif size < (1024 * 1024 * 1024):
        return ("%.2f MB") % (float(size) / (1024 * 1024))
    else:
        return ("%.2f GB") % (float(size) / (1024 * 1024 * 1024))

########################################## 
class RunId(genericColumn):
   path=""
   colName='Run ID'
   attrNames=["run_id"]
   def decorator(self,values):
    return """<a href="run?run_id=%s">%s</a>"""%(values[0],values[0])
   
class JobId(genericColumn):
   colName='Job ID'
   attrNames=["job_id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Jobs&Id=%s\">%s</a>"%(values[0],values[0])
   
class FileId(JobId):
   colName='Id'
   attrNames=["id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Files&Id=%s\">%s</a>"%(values[0],values[0])
   
class StreamerId(JobId):
   colName='Id'
   attrNames=["streamer_id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Files&Id=%s\">%s</a>"%(values[0],values[0])
        #,filesize(),events(),dataset(),exportable(),lfn()

class lumiId(genericColumn):
   colName='Lumi Id'
   attrNames=["lumi_id"]
   
class nstreamers(genericColumn):
   colName='Streamers'
   attrNames=["num_streamers"]
   
class stream(genericColumn):
   colName='Stream'
   attrNames=["stream"]
      
class tier(genericColumn):
   colName='Tier'
   attrNames=["tier"]      
   
class skim(genericColumn):
   colName='Skim'
   attrNames=["skim"]
         
   
class dataset(genericColumn):
   colName='Dataset'
   attrNames=["dataset"]
         
class sTime(genericColumn):
   colName='Start'
   attrNames=["start_time"]
   def decorator(self,values):
     str= """ &nbsp;<a class="info" href="#"><img src="imgs/clock.png" alt="" style="border: 0px none ;"><span style="width: 130px; left: 0px;">%s</span></a>"""
     return str%(strftime("%Y-%m-%d %H:%M:%S", gmtime(int(values[0]))))
   
class eTime(genericColumn):
   colName='End'
   attrNames=["end_time"]
   def decorator(self,values):
    str= """ &nbsp;<a class="info" href="#"><img src="imgs/clock.png" alt="" style="border: 0px none ;"><span style="width: 130px; left: 0px;">%s</span></a>"""
    return str%(strftime("%Y-%m-%d %H:%M:%S", gmtime(int(values[0]))))
   
class defTime(genericColumn):
   colName='Definition Time'
   attrNames=["def_time"]
   def decorator(self,values):
      return strftime("%Y-%m-%d %H:%M:%S", gmtime(int(values[0])))

class runTime(genericColumn):
   colName='Running Time'
   attrNames=["state_time"]
   def decorator(self,values):
    kk=mktime(localtime())-int(values[0])
    return strftime("%Hh %Mm %Ss", gmtime(kk))
   
class filesNum(genericColumn):
   colName='# of files'
   attrNames=["inputfiles"]
   
class filesize(genericColumn):
   colName='Filesize'
   attrNames=["filesize"]
   def decorator(self,values):
    return self.humanFilesize(int(values[0]))  
 
class exportable(genericColumn):
   colName='Exportable'
   attrNames=["streamer_exp"]
         
class process(genericColumn):
   colName='Process'
   attrNames=["process"]
  
class version(genericColumn):
   colName='Version'
   attrNames=["version"]
   def decorator(self,values): 
     return str(values[0].replace("CMSSW_","")) 
    
class expVersion(genericColumn):
   colName='Express Version'
   attrNames=["expversion"]
   def decorator(self,values): 
     return str(values[0].replace("CMSSW_","")) 
    
   
class repVersion(genericColumn):
   colName='Repack Version'
   attrNames=["repversion"]
   def decorator(self,values): 
     return str(values[0].replace("CMSSW_","")) 
    

class hlt(genericColumn):
   colName='HLTKey'
   attrNames=["hltkey"]
   def decorator(self,values):
    l=len(values[0])
    if l>10: string="..."+values[0][l-10:l]
    else: string=values[0]
    return  """<a href="/%s">%s</a>"""%(values[0],string)
    
   
class acq(genericColumn):
   colName='ACQ Era'
   attrNames=["acq_era"]

class status(genericColumn):
   colName='Status'
   attrNames=["status"]
   
class events(genericColumn):
   colName='Events'
   attrNames=["events"] 
     
class lfn(genericColumn):
   colName='Lfn'
   attrNames=["lfn"]
   def decorator(self,values):
    return  "<a href=\"/%s\"><img src=\"imgs/doc.png\"  alt=\"\" border=\"0\" height=\"12\"></a>"""%(values[0])

class splitInProgress(genericColumn):
   colName="Split in Progress"
   attrNames=["splitinprocessing"]
   
class procVersion(genericColumn):
   colName="Proc Version"
   attrNames=["proc_version"]
   
class globalTag(genericColumn):
   colName="Global Tag"
   attrNames=["global_tag"]
   
class confUrl(genericColumn):
   colName="Config  Url"
   attrNames=["config_url"]
   def decorator(self,values):
    #print values[0]
    l=len(values[0])
    if values[0]!="None": 
      if l>40: string="..."+values[0][l-40:l]
      else: string=values[0]
    else: return values[0]
    return  """<a href="%s">%s</a>"""%(values[0],string)
   
  
####################################################

#values = list with attrNames + auxInfo
class graphColumn(genericColumn):
  option="?run=%s"
  MAXLEN=0.8
 
  def decorator(self,values):
    if len(values)==1: return str(values[0])
    else: values=[int(v) for v in values]
    percentuals=self.getPercentuals(values)
    #return str(values) +" <br/> " +percentuals
    return percentuals
    
  def getAuxInfo(self,table,run_dict):
    runid=str(run_dict[RunId().attrNames[0]])
    res=[]
    for i in self.auxSuffixes:
      #print i + "   "+table.DASUrl+i+self.option%(runid)
      valueDict=table.fetcher(table.DASUrl+i+self.option%(runid))
      #print "OUTPUT=" +str(valueDict)
      value = valueDict[0]['count']
      #print "OUTPUT=" +str(value)
      res.append(value)
      run_dict[i]=value  
    return res
    
  def printPercentual(self,num,denom):
    if denom>0:
      perc= ceil(100*num//denom)
    else: perc=0
    if perc>100:
      perc=100
      imageName="imgs/barR.png"
    elif perc==100:
      imageName="imgs/barG.png"
    elif perc>0:
      imageName="imgs/barB.png"
    else:imageName="imgs/barR.png"
    res="<img src=\""+imageName+"\"  alt=\"\" height=\"12\" width=\"%d\"> %d /%d = %d%% "%(ceil(self.MAXLEN*perc), num,denom,perc )
       ##print percentuals
    res+="<br/>"
    return res
    
  def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printPercentual(values[0],values[1])
     percentuals+=self.printPercentual(values[2],values[3])
     percentuals+=self.printPercentual(values[4],values[5])
    except:
      pass
    return percentuals
################################################################################# FIXME: change the names of the variables!!! no debug possible otherwise
   
class numstreamers(graphColumn):
   colName='Streamers'
   attrNames=['numstreamers']
   auxSuffixes=[]
   

class expressGraph(graphColumn):
   colName='Express Graph'
   attrNames=['numexprcompleted','numexpresstotal','numexpressmerged','numexpressmergetotal']
   #'numexprstreamers',
   auxSuffixes=['numexpresstotalstreamers','numexpressstreamers']
   def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printPercentual(values[5],values[4])
     percentuals+=self.printPercentual(values[0],values[1])
     percentuals+=self.printPercentual(values[2],values[3])
    except:
      pass
    return percentuals

class repackGraph(graphColumn):
   colName='Repacked Graph'
   attrNames=['numrepacktemp','numstreamers','numexpresstotalstreamers','numrepackcompleted','numrepackedtotal','numrepackmergedexported','numrepackmerged']
   auxSuffixes=[]
   
   def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printPercentual(values[0],values[1]-values[2])
     percentuals+=self.printPercentual(values[3],values[4])
     percentuals+=self.printPercentual(values[5],values[6])
    except:
      pass
    return percentuals
class recoGraph(graphColumn):
   colName='Reconstructed Graph'
   attrNames=['numrecotemp','numrecotemptotal','numrecocompleted','numrecototal','numrecomergedexported','numrecomerged']
   auxSuffixes=[]
   
     
class alcaGraph(recoGraph):
   colName='Alcaskimmed Graph'
   auxSuffixes=[]
   attrNames=['numalcatemp','numalcatemptotal','numalcacompleted','numalcatotal','numalcamergedexported','numalcamerged']
    
